@@ -314,7 +314,8 @@ add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType fold
 			ESource *source = temp_list->data;
 
 			if (source && E_IS_SOURCE (source)) {
-				if (strcmp (e_source_get_property (source, "public"), "yes") != 0)
+				if (!e_source_get_property (source, "public") ||
+				    strcmp (e_source_get_property (source, "public"), "yes") != 0)
 					e_source_group_remove_source (group, source);
 			}
 		}
@@ -714,7 +715,8 @@ add_addressbook_sources (EAccount *account, GSList *folders, mapi_id_t trash_fid
 			ESource *source = temp_list->data;
 
 			if (source && E_IS_SOURCE (source)) {
-				if (strcmp(e_source_get_property(source, "public"), "yes") != 0)
+				if (!e_source_get_property (source, "public") ||
+				    strcmp (e_source_get_property (source, "public"), "yes") != 0)
 					e_source_group_remove_source (group, source);
 			}
 		}
