@@ -236,10 +236,10 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 	/*Can there be a account without password ?*/
 	if (password && *password && domain_name && *domain_name && *url->user && *url->host) {
 		guint32 cp_flags = (camel_url_get_param (url, "ssl") && g_str_equal (camel_url_get_param (url, "ssl"), "1")) ? CREATE_PROFILE_FLAG_USE_SSL : CREATE_PROFILE_FLAG_NONE;
-		gchar *error_msg = NULL;
-		gboolean status = exchange_mapi_create_profile (url->user, password, domain_name,
-								url->host, cp_flags, &error_msg,
-								(mapi_profile_callback_t) create_profile_callback, url->user);
+		GError *error = NULL;
+		gboolean status = exchange_mapi_create_profile (url->user, password, domain_name, url->host, cp_flags,
+								(mapi_profile_callback_t) create_profile_callback, url->user,
+								&error);
 		if (status) {
 			/* profile was created, try to connect to the server */
 			ExchangeMapiConnection *conn;
@@ -248,7 +248,7 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 			status = FALSE;
 			profname = exchange_mapi_util_profile_name (url->user, domain_name, url->host, FALSE);
 
-			conn = exchange_mapi_connection_new (profname, password);
+			conn = exchange_mapi_connection_new (profname, password, &error);
 			if (conn) {
 				status = exchange_mapi_connection_connected (conn);
 				g_object_unref (conn);
@@ -276,14 +276,15 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 
 			e_passwords_forget_password (EXCHANGE_MAPI_PASSWORD_COMPONENT, key);
 
-			e = g_strconcat (_("Authentication failed."), "\n", error_msg, NULL);
+			e = g_strconcat (_("Authentication failed."), "\n", error ? error->message : NULL, NULL);
 
 			e_notice (NULL, GTK_MESSAGE_ERROR, "%s", e);
 
 			g_free (e);
 		}
 
-		g_free (error_msg);
+		if (error)
+			g_error_free (error);
 	} else {
 		e_passwords_forget_password (EXCHANGE_MAPI_PASSWORD_COMPONENT, key);
 		e_notice (NULL, GTK_MESSAGE_ERROR, "%s", _("Authentication failed."));
@@ -610,7 +611,8 @@ static GtkWidget *
 exchange_mapi_create (GtkWidget *parent, ESource *source, ExchangeMAPIFolderType folder_type)
 {
 	GtkWidget *vbox, *label, *scroll, *tv;
-	gchar *uri_text;
+	gchar *uri_text, *profile = NULL;
+	ESourceGroup *group;
 	gint row;
 	GtkCellRenderer *rcell;
 	GtkTreeStore *ts;
@@ -626,11 +628,17 @@ exchange_mapi_create (GtkWidget *parent, ESource *source, ExchangeMAPIFolderType
 	}
 
 	folders = NULL;
-	conn = exchange_mapi_connection_find (e_source_get_property (source, "profile"));
+	group = e_source_peek_group (source);
+	profile = g_strdup (e_source_get_property (source, "profile"));
+	if (profile == NULL) {
+		profile = e_source_group_get_property (group, "profile");
+		e_source_set_property (source, "profile", profile);
+	}
+	conn = exchange_mapi_connection_find (profile);
+	g_free (profile);
 	if (conn && exchange_mapi_connection_connected (conn))
 		folders = exchange_mapi_connection_peek_folders_list (conn);
-
-	acc = e_source_group_peek_name (e_source_peek_group (source));
+	acc = e_source_group_peek_name (group);
 	ts = gtk_tree_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_INT64, G_TYPE_POINTER);
 
 	add_folders (folders, ts, folder_type);
@@ -743,13 +751,40 @@ exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 {
 	EABConfigTargetSource *t = (EABConfigTargetSource *) target;
 	ESource *source = t->source;
-	gchar *uri_text;
+	gchar *uri_text, *r_uri, *sfid;
 	ESourceGroup *grp;
+	ExchangeMapiConnection *conn;
+	mapi_id_t fid, pfid;
+	GError *mapi_error = NULL;
 
 	uri_text = e_source_get_uri (source);
 	if (uri_text && g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH))
 		return;
 
+	exchange_mapi_util_mapi_id_from_string (e_source_get_property (source, "parent-fid"), &pfid);
+
+	/* the profile should be already connected */
+	conn = exchange_mapi_connection_find (e_source_get_property (source, "profile"));
+	g_return_if_fail (conn != NULL);
+
+	fid = exchange_mapi_connection_create_folder (conn, olFolderContacts, pfid, 0, e_source_peek_name (source), &mapi_error);
+	g_object_unref (conn);
+
+	if (!fid) {
+		if (mapi_error) {
+			e_notice (NULL, GTK_MESSAGE_ERROR, _("Failed to create address book '%s': %s"), e_source_peek_name (source), mapi_error->message);
+			g_error_free (mapi_error);
+		} else {
+			e_notice (NULL, GTK_MESSAGE_ERROR, _("Failed to create address book '%s'"), e_source_peek_name (source));
+		}
+
+		return;
+	}
+
+	sfid = exchange_mapi_util_mapi_id_to_string (fid);
+	r_uri = g_strconcat (";", sfid, NULL);
+	e_source_set_relative_uri (source, r_uri);
+
 	//FIXME: Offline handling
 	grp = e_source_peek_group (source);
 	e_source_set_property (source, "auth", "plain/password");
@@ -761,7 +796,10 @@ exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 	e_source_set_relative_uri (source, g_strconcat (";",e_source_peek_name (source), NULL));
 
 	e_source_set_property (source, "completion", "true");
+	e_source_set_property (source, "public", "no");
 	// Update the folder list in the plugin and ExchangeMAPIFolder
+	g_free (r_uri);
+	g_free (sfid);
 
 	return;
 }
@@ -807,6 +845,7 @@ exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 	mapi_id_t fid, pfid;
 	uint32_t type;
 	gchar *uri_text = e_source_get_uri (source);
+	GError *mapi_error = NULL;
 
 	if (!uri_text || g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH))
 		return;
@@ -835,9 +874,20 @@ exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 	conn = exchange_mapi_connection_find (e_source_get_property (source, "profile"));
 	g_return_if_fail (conn != NULL);
 
-	fid = exchange_mapi_connection_create_folder (conn, type, pfid, e_source_peek_name (source));
+	fid = exchange_mapi_connection_create_folder (conn, type, pfid, 0, e_source_peek_name (source), &mapi_error);
 	g_object_unref (conn);
 
+	if (!fid) {
+		if (mapi_error) {
+			e_notice (NULL, GTK_MESSAGE_ERROR, _("Failed to create calendar '%s': %s"), e_source_peek_name (source), mapi_error->message);
+			g_error_free (mapi_error);
+		} else {
+			e_notice (NULL, GTK_MESSAGE_ERROR, _("Failed to create calendar '%s'"), e_source_peek_name (source));
+		}
+
+		return;
+	}
+
 	sfid = exchange_mapi_util_mapi_id_to_string (fid);
 	tmp = g_strconcat (";", sfid, NULL);
 	e_source_set_relative_uri (source, tmp);
@@ -847,6 +897,7 @@ exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 	e_source_set_property (source, "auth", "1");
 	e_source_set_property (source, "auth-domain", EXCHANGE_MAPI_PASSWORD_COMPONENT);
 	e_source_set_property (source, "auth-type", "plain/password");
+	e_source_set_property (source, "public", "no");
 
 	group = e_source_peek_group (source);
 
