@@ -299,6 +299,7 @@ ebbm_update_cache_cb (gpointer data)
 	EBookBackendMAPIClass *ebmac;
 	glong last_modification_secs = 0;
 	EDataBookView *book_view;
+	GError *error = NULL;
 
 	g_return_val_if_fail (ebma != NULL, NULL);
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma), NULL);
@@ -324,14 +325,17 @@ ebbm_update_cache_cb (gpointer data)
 		if (!ebbm_get_cache_time (ebma, &last_modification_secs))
 			last_modification_secs = 0;
 
-		ebbm_fetch_contacts (ebma, restriction, NULL, &last_modification_secs, NULL);
+		ebbm_fetch_contacts (ebma, restriction, NULL, &last_modification_secs, &error);
+		if (!error)
+			e_book_backend_cache_set_populated (priv->cache);
+		else
+			e_file_cache_remove_object (E_FILE_CACHE (priv->cache), "populated");
 
 		talloc_free (mem_ctx);
 	}
 
-	if (!g_cancellable_is_cancelled (priv->update_cache) && ebmac->op_fetch_known_uids) {
+	if (!error && !g_cancellable_is_cancelled (priv->update_cache) && ebmac->op_fetch_known_uids) {
 		GHashTable *uids = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free, NULL);
-		GError *error = NULL;
 
 		ebmac->op_fetch_known_uids (ebma, priv->update_cache, uids, &error);
 
@@ -347,7 +351,8 @@ ebbm_update_cache_cb (gpointer data)
 
 				if (!uid || g_hash_table_lookup (uids, uid)
 				    || g_str_equal (uid, "populated")
-				    || g_str_equal (uid, "last_update_time"))
+				    || g_str_equal (uid, "last_update_time")
+				    || g_str_has_prefix (uid, "key:"))
 					continue;
 
 				/* uid is hold by a cache, thus make a copy before remove */
@@ -366,10 +371,11 @@ ebbm_update_cache_cb (gpointer data)
 		}
 
 		g_hash_table_destroy (uids);
-		if (error)
-			g_error_free (error);
 	}
 
+	if (error)
+		g_error_free (error);
+
 	book_view = ebbm_pick_book_view (ebma);
 	if (book_view)
 		e_data_book_view_notify_complete (book_view, NULL);
@@ -693,7 +699,7 @@ ebbm_book_view_thread (gpointer data)
 
 	e_book_backend_mapi_update_view_by_cache (bvtd->ebma, bvtd->book_view, &error);
 
-	if (!error && priv && priv->conn && !priv->update_cache_thread
+	if (!error && priv && priv->conn && (!priv->update_cache_thread || g_cancellable_is_cancelled (priv->update_cache))
 	    && e_book_backend_mapi_book_view_is_running (bvtd->ebma, bvtd->book_view)) {
 		EBookBackendMAPIClass *ebmac;
 
@@ -701,12 +707,20 @@ ebbm_book_view_thread (gpointer data)
 		if (ebmac && ebmac->op_book_view_thread)
 			ebmac->op_book_view_thread (bvtd->ebma, bvtd->book_view, &error);
 
-		if (!error && !priv->marked_for_offline) {
+		if (!error && !e_book_backend_cache_is_populated (priv->cache)) {
 			/* todo: create restriction based on the book_view */
+			g_cancellable_reset (priv->update_cache);
 			ebbm_fetch_contacts (bvtd->ebma, NULL, bvtd->book_view, NULL, &error);
+			g_cancellable_cancel (priv->update_cache);
+
+			if (!error)
+				e_book_backend_cache_set_populated (priv->cache);
 		}
 	}
 
+	if (error && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+		g_clear_error (&error);
+
 	/* do not stop book view when filling cache */
 	if (e_book_backend_mapi_book_view_is_running (bvtd->ebma, bvtd->book_view)
 	    && (!priv->update_cache_thread || g_cancellable_is_cancelled (priv->update_cache)))
@@ -1624,6 +1638,54 @@ e_book_backend_mapi_notify_contact_removed (EBookBackendMAPI *ebma, const gchar
 	e_book_backend_notify_remove (E_BOOK_BACKEND (ebma), uid);
 }
 
+static gchar *
+create_cache_key (const gchar *key)
+{
+	g_return_val_if_fail (key != NULL, NULL);
+
+	return g_strconcat ("key:", key, NULL);
+}
+
+void
+e_book_backend_mapi_cache_set (EBookBackendMAPI *ebma, const gchar *key, const gchar *value)
+{
+	gchar *real_key;
+
+	g_return_if_fail (ebma != NULL);
+	g_return_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma));
+	g_return_if_fail (ebma->priv != NULL);
+	g_return_if_fail (ebma->priv->cache != NULL);
+	g_return_if_fail (key != NULL);
+
+	real_key = create_cache_key (key);
+	g_return_if_fail (real_key != NULL);
+
+	if (!e_file_cache_add_object (E_FILE_CACHE (ebma->priv->cache), real_key, value))
+		e_file_cache_replace_object (E_FILE_CACHE (ebma->priv->cache), real_key, value);
+
+	g_free (real_key);
+}
+
+gchar *
+e_book_backend_mapi_cache_get (EBookBackendMAPI *ebma, const gchar *key)
+{
+	gchar *real_key, *res;
+
+	g_return_val_if_fail (ebma != NULL, NULL);
+	g_return_val_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma), NULL);
+	g_return_val_if_fail (ebma->priv != NULL, NULL);
+	g_return_val_if_fail (ebma->priv->cache != NULL, NULL);
+	g_return_val_if_fail (key != NULL, NULL);
+
+	real_key = create_cache_key (key);
+	g_return_val_if_fail (real_key != NULL, NULL);
+
+	res = g_strdup (e_file_cache_get_object (E_FILE_CACHE (ebma->priv->cache), real_key));
+	g_free (real_key);
+
+	return res;
+}
+
 /* utility functions/macros */
 
 /* 'data' is one of GET_ALL_KNOWN_IDS or GET_UIDS_ONLY */
@@ -1852,15 +1914,10 @@ mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid,
 				e_contact_set (contact, mappings[i].field_id, value);
 		} else if (contact_type == ELEMENT_TYPE_SIMPLE) {
 			if (value && mappings[i].element_type == PT_SYSTIME) {
-				struct FILETIME *t = value;
-				time_t time;
-				NTTIME nt;
+				const struct FILETIME *t = value;
 				gchar buff[129];
+				time_t time = exchange_mapi_util_filetime_to_time_t (t);
 
-				nt = t->dwHighDateTime;
-				nt = nt << 32;
-				nt |= t->dwLowDateTime;
-				time = nt_time_to_unix (nt);
 				e_contact_set (contact, mappings[i].field_id, ctime_r (&time, buff));
 			}
 		} else if (contact_type == ELEMENT_TYPE_COMPLEX) {
@@ -1872,18 +1929,15 @@ mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid,
 				g_list_free (list);
 			} else if (mappings[i].field_id == E_CONTACT_BIRTH_DATE
 				   || mappings[i].field_id == E_CONTACT_ANNIVERSARY) {
-				struct FILETIME *t = value;
+				const struct FILETIME *t = value;
 				time_t time;
-				NTTIME nt;
 				struct tm * tmtime;
 				if (value) {
 					EContactDate date = {0};
-					nt = t->dwHighDateTime;
-					nt = nt << 32;
-					nt |= t->dwLowDateTime;
-					time = nt_time_to_unix (nt);
+
+					time = exchange_mapi_util_filetime_to_time_t (t);
 					tmtime = gmtime (&time);
-					//FIXME: Move to new libmapi api to get string dates.
+
 					date.day = tmtime->tm_mday;
 					date.month = tmtime->tm_mon + 1;
 					date.year = tmtime->tm_year + 1900;
